package miya.epayment

import com.alibaba.fastjson.{JSON, JSONArray}
import org.json.{JSONObject, XML}

import scala.util.control.NonFatal


/**
  * Created by hope on 2018/8/31.
  */
object Tools {

  //模式匹配支付code简称对应的支付名全称
  def matchPaymentPlatform(x: String): String = x match {
    case "1" => "微信"
    case "2" => "翼支付3.0"
    case "D" => "支付宝"
    case "3" => "支付宝"
    case "K" => "工行"
    case "4" => "百度钱包"
    case "5" => "翼支付"
    case "6" => "手Q"
    case "7" => "银商资讯"
    case "8" => "微众钱包"
    case "A" => "大众点评"
    case "B" => "银商预付卡"
    case "C" => "京东钱包"
    case "E" => "招行"
    case "I" => "百糯"
    case "L" => "飞凡"
    case "M" => "华润银行"
    case "N" => "银联二维码"
    case "P" => "京东钱包"
    case "T" => "润钱包"
    case "U" => "移动和包"
    case "S" => "汇付天下"
    case "Q" => "银联商务"
    case "9" => "微众有折"
    case "F" => "支付宝国际化"
    case "H" => "微信国际化"
    case "J" => "微信香港钱包"
    case "W" => "网商银行"
    case "Y" => "沃支付"
    case "R" => "京东3.0"
    case "G" => "屈臣氏虚拟卡"
    case _ => "未知"
  }

  def distinctStr(str: String): String = {
    try {
      val lastStr = str.split("-").filter(!"null".equals(_)).last
      lastStr
    } catch {
      case e: Exception => {
        null
      }
    }
  }

  def str2Arr(str: String): Array[String] = {
    var strArray: Array[String] = null
    try {
      strArray = str.split(",")
      strArray
    } catch {
      case e: Exception => {
        strArray
      }
    }
  }

  def str2DoubleArr(str: String): Array[Double] = {
    var doubleArray: Array[Double] = null
    try {
      doubleArray = Array[Double](str.toDouble)
      doubleArray
    } catch {
      case e: Exception => {
        doubleArray
      }
    }
  }

  def str2Double(str: String): Double = {
    try {
      str.toDouble
    } catch {
      case e: Exception => {
        0
      }
    }
  }

  def str2Long(str: String): Long = {
    try {
      str.toLong
    } catch {
      case e: Exception => {
        0
      }
    }
  }

  /**
    * 把优惠信息里面的数据加工成实收金额,实收=实扣+平台优惠
    * 优惠金额（格式：[实扣金额|商户优惠金额|平台优惠金额|预留|预留|预留]）
    * 例子:[1500|0|0|0|0|0]
    *
    * @param promoteAmt
    *
    */
  def getActualReceived(promoteAmt: String, orderAmt: String): Long = {
    try {
      if ("[0|0|0|0|0|0]".equals(promoteAmt)) {
        orderAmt.toLong
      } else {
        val promoteAmtArr: Array[String] = promoteAmt.replace("[", "").replace("]", "").split("[|]")
        promoteAmtArr(0).toLong + promoteAmtArr(2).toLong
      }
    } catch {
      case NonFatal(t) => {
        try {
          orderAmt.toLong
        } catch {
          case NonFatal(t) => 0
        }
      }
    }
  }


  /**
    * 把优惠信息里面的数据加工成实收金额,实收=实扣+平台优惠
    * 优惠金额（格式：[实扣金额|商户优惠金额|平台优惠金额|预留|预留|预留]）
    * 例子:[1500|0|0|0|0|0]
    *
    * @param promoteAmt
    * @param orderAmt
    *
    */
  def getTotalAmt(promoteAmt: String, orderAmt: String): String = {
    try {

      //        println("promoteAmt: " + promoteAmt)
      //        println("orderAmt: " + orderAmt)
      //        println("------------------------------")
      if ("[0|0|0|0|0|0]".equals(promoteAmt) || promoteAmt.contains("bill")) {
        orderAmt
      } else {
        val promoteAmtArr: Array[String] = promoteAmt.replace("[", "").replace("]", "").split("[|]")
        promoteAmtArr.map(_.toLong).sum.toString
      }


    } catch {
      case NonFatal(t) => {
        try {
          orderAmt
        } catch {
          case NonFatal(t) => null
        }
      }
    }
  }


  /**
    * 把优惠信息里面的数据计算相应的优惠金额
    * 优惠金额（格式：[实扣金额|商户优惠金额|平台优惠金额|预留|预留|预留]）
    * 例子:[1500|0|0|0|0|0]
    *
    * @param promoteAmt  [1500|0|0|0|0|0]
    * @param promoteType 优惠类型:1-商户优惠金额,2-平台优惠金额
    *
    */
  def getPromoteInfo(promoteAmt: String, promoteType: String): String = {
    try {
      if (null == promoteAmt || null == promoteType) {
        null
      } else {
        if (promoteAmt.contains("bill")) {
          //consumer_bill:1190;saas_bill:0;other_bill:0
          val billArr: Array[String] = promoteAmt.split(";")
          if ("2" == promoteType) {
            val merchantPromoteAmt = billArr.filter(t => t.contains("other_bill")).last.split(":")(1)
            if (merchantPromoteAmt.contains("null")) null else merchantPromoteAmt
          } else {
            val platformPromoteAmt = billArr.filter(t => t.contains("saas_bill")).last.split(":")(1)
            if (platformPromoteAmt.contains("null")) null else platformPromoteAmt
          }
        } else {
          val promoteInfo = promoteAmt.replace("[", "").replace("]", "").split("[|]")(promoteType.toInt)
          if (promoteInfo.contains("null")) null else promoteInfo
        }
      }
    } catch {
      case NonFatal(t) => {
        try {
          null
        } catch {
          case NonFatal(t) => null
        }
      }
    }
  }


  /**
    * 把原始日志的数据加工成总金额
    * 优惠金额（格式：[实扣金额|商户优惠金额|平台优惠金额|预留|预留|预留]）
    * 例子:[1500|0|0|0|0|0]
    *
    * @param xmlLog
    *
    */
  def getTotalAmt(xmlLog: String): Long = {
    try {
      val json = XML.toJSONObject(xmlLog)
      val xmlObj = json.getJSONObject("xml")
      //      println(xmlObj)
      val promoteAmt = xmlObj.get("C10").toString
      //println(promoteAmt)
      val orderAmt = xmlObj.get("C7").toString
      //println("orderAmt"+orderAmt)
      if (null != promoteAmt && "" != promoteAmt && !"[0|0|0|0|0|0]".equals(promoteAmt)) {
        val promoteAmtArr: Array[String] = promoteAmt.replace("[", "").replace("]", "").split("[|]")
        promoteAmtArr.map(_.toLong).sum
      } else if (null != orderAmt && "" != orderAmt) {
        orderAmt.toLong
      } else {
        0
      }


    } catch {
      case NonFatal(t) => {
        0
      }
    }
  }


  /**
    * 把原始日志中的b5单品详情,求出单品数量
    *
    * @param b5
    *
    */
  def getGoodsAmt(b5: String): Double = {
    try {
      val goodsArr: JSONArray = JSON.parseArray(b5)
      var tatal: Double = 0
      for (i <- 0 until (goodsArr.size())) {
        val quantity = goodsArr.getJSONObject(i).getString("quantity")
        var num: Double = 0
        if (!quantity.endsWith("0")) {
          num = 1.0
        } else {
          num = quantity.toDouble

        }
        tatal += num
      }
      tatal
    } catch {
      case e: Exception => {
        0
      }
    }
  }


  /**
    * 把原始日志中的userid取出來,微信c9
    * 支付寶c19
    *
    * @param trade03Msg
    *
    */
  def getUserid(trade03Msg: JSONObject): String = {
    try {
      if (trade03Msg.toString.contains("微信")) {
        JsonUtils.jsonObjectGetString(trade03Msg, "C9")
      } else {
        JsonUtils.jsonObjectGetString(trade03Msg, "C19")
      }
    } catch {
      case NonFatal(t) => {
        null
      }
    }
  }


  def main(args: Array[String]): Unit = {
    //    println(getTotalAmt("[5748|177|0|0|0|0]"))
    //    println(getTotalAmt("[5748|177|0|0|0|0]"))
    //    println(getTotalAmt("[1500|0|0|0|0|0]"))
    //    println(getTotalAmt("", ""))
    //    println(getTotalAmt("{\"status\":\"00\",\"trade_no\":\"wx2817340979719447c99b520d1813199200\",\"wxsend\":{\"appId\":\"wx9443c4ef422fa1fa\",\"timeStamp\":\"1561714449835\",\"signType\":\"MD5\",\"package\":\"prepay_id=wx2817340979719447c99b520d1813199200\",\"nonceStr\":\"owJkGYTgfkqfXhBCvC8hNZ0RgSzEbgEo\",\"paySign\":\"361C5C0044E557134222A57CADC33473\"},\"payserial\":\"A30Q190628Q601004301020100\"}", "400"))
    //    println(getTotalAmt(null, "10840"))
    //    println(matchPaymentPlatform(distinctStr("B-A")))
    //    println(matchPaymentPlatform(distinctStr("B-B-B-B")))
    //    println(matchPaymentPlatform(distinctStr("B")))
    //println(getGoodsAmt("[{\"goodsId\":\"11002177\",\"goodsName\":\"芒果干\",\"price\":\"159.8\",\"quantity\":\"2.143\"},{\"goodsId\":\"80000111\",\"goodsName\":\"中号纸提袋（常规版）\",\"price\":\"0.5\",\"quantity\":\"1.0\"},{\"goodsId\":\"11005934\",\"goodsName\":\"黄桃汁\",\"price\":\"9.8\",\"quantity\":\"1.0\"},{\"goodsId\":\"11002493\",\"goodsName\":\"黄桃果捞（糖水黄桃罐头）\",\"price\":\"7.9\",\"quantity\":\"2.0\"}]"))
    //    println(getTotalAmt("consumer_bill:8350;saas_bill:0;other_bill:0;brand_bill:0", "8350"))
    println(getTotalAmt("[1480|0|0|0|0|0]", null))

  }
}
